/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.geronimo.samples.inventory.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.geronimo.samples.inventory.Item;
import org.apache.geronimo.samples.inventory.exception.ItemException;


public class ItemDAO {

    private final DataSource ds;

    public ItemDAO() throws ItemException {
        try {
            InitialContext ctx = new InitialContext();
            ds = (DataSource) ctx.lookup("java:comp/env/jdbc/InventoryDS");
        } catch (NamingException e) {
            throw new ItemException(e);
        }
    }

    public List<Item> getItems() throws ItemException {
        ArrayList<Item> items = new ArrayList<Item>();

        try {
            Connection con = ds.getConnection();
            try {

                PreparedStatement pStmt = con.prepareStatement("SELECT i.item_id as item_id, i.item_name as item_name, i.description as description, im.quantity as quantity FROM item i, item_master im WHERE im.item_id = i.item_id");
                try {
                    ResultSet rs = pStmt.executeQuery();
                    try {
                        while (rs.next()) {
                            Item item = new Item();

                            item.setItemNo(rs.getString("item_id"));
                            item.setItemName(rs.getString("item_name"));
                            item.setDescription(rs.getString("description"));
                            item.setQuantity(rs.getInt("quantity"));


                            items.add(item);
                        }
                    } finally {
                        rs.close();
                    }
                } finally {
                    pStmt.close();
                }

            } finally {
                con.close();
            }
        } catch (SQLException e) {
            throw new ItemException(e);
        }

        return items;
    }

    public int getQOH(String itemNo) throws ItemException {

        int qoh = 0;
        try {
            Connection con = ds.getConnection();
            try {

                PreparedStatement pStmt = con.prepareStatement("SELECT quantity FROM item_master WHERE item_id =? ");
                try {
                    pStmt.setString(1, itemNo);

                    ResultSet rs = pStmt.executeQuery();
                    try {
                        if (rs.next()) {
                            qoh = rs.getInt("quantity");
                        }
                    } finally {
                        rs.close();
                    }
                } finally {
                    pStmt.close();
                }
            } finally {
                con.close();
            }
        } catch (SQLException e) {
            throw new ItemException(e);
        }
        return qoh;
    }

    public boolean isItemIdExists(String itemNo) throws ItemException {
        boolean isExists = false;
        try {
            Connection con = ds.getConnection();
            try {

                PreparedStatement pStmt = con.prepareStatement("SELECT item_id FROM item WHERE item_id =? ");
                try {
                    pStmt.setString(1, itemNo);

                    ResultSet rs = pStmt.executeQuery();
                    try {
                        if (rs.next()) {
                            isExists = true;
                        }
                    } finally {
                        rs.close();
                    }
                } finally {
                    pStmt.close();
                }
            } finally {
                con.close();
            }
        } catch (SQLException e) {
            throw new ItemException(e);
        }
        return isExists;
    }

    public void updateQOH(String itemNo, int qoh) throws ItemException {
        try {
            Connection con = ds.getConnection();
            try {

                PreparedStatement pStmt = con.prepareStatement("UPDATE item_master SET quantity=? WHERE item_id=?");
                try {
                    pStmt.setInt(1, qoh);
                    pStmt.setString(2, itemNo);

                    pStmt.executeUpdate();
                } finally {
                    pStmt.close();
                }
            } finally {
                con.close();
            }
        } catch (SQLException e) {
            throw new ItemException(e);
        }
    }

    public void addItem(Item item) throws ItemException {
        try {
            Connection con = ds.getConnection();
            try {

                PreparedStatement pStmt = con.prepareStatement("INSERT INTO item VALUES(?, ?, ?)");
                try {
                    pStmt.setString(1, item.getItemNo());
                    pStmt.setString(2, item.getItemName());
                    pStmt.setString(3, item.getDescription());

                    pStmt.executeUpdate();

                    pStmt = con.prepareStatement("INSERT INTO item_master VALUES(?, ?)");
                    pStmt.setString(1, item.getItemNo());
                    pStmt.setInt(2, 0);

                    pStmt.executeUpdate();
                } finally {
                    pStmt.close();
                }

            } finally {
                con.close();
            }
        } catch (SQLException e) {
            throw new ItemException(e);
        }
    }

}
